#include <rgb_lcd.h>
#include <Keypad.h>

/* 
Juego Hi & Low

Vamos a tener los siguientes estados:
Estado 0: Pantalla de inicio y se genera el número aleatorio entre 0 y 1000
Estado 1: el usuario ingresa un número de tres dígitos como máximo
Estado 2: Arduino compara el número ingresado con el escondido y da los resultados
Estado 3: Usuario adivina el número
Estado 4: Usuario se rinde y se muestra el número que había que adivinar

*/

/******* KEYPAD *********/ 
const byte ROWS = 4; //four rows
const byte COLS = 4; //four columns

//matriz de mapeo del teclado
char keys[ROWS][COLS] = {
  {'1','2','3','A'},
  {'4','5','6','B'},
  {'7','8','9','C'},
  {'*','0','#','D'}
};

//conexión de filas y columnas del keypad al Arduino
byte rowPins[ROWS] = {13, 12, 11, 10}; //connect to the row pinouts of the keypad
byte colPins[COLS] = {8, 7, 6, 5}; //connect to the column pinouts of the keypad

// creación del objeto teclado
Keypad teclado = Keypad( makeKeymap(keys), rowPins, colPins, ROWS, COLS );

/******* PANTALLA LDC *********/
rgb_lcd pantalla;

/******* VARIABLES PARA EL JUEGO ********/
int estado;
int hiddenNumber, userNumber;
int intentos;
String ingreso;  //números como caracteres que ingresa el usuario
int digitosIngresados;

/******** BOTON *********/
int boton = 2;
int valorBoton;

void inicializar(){
  estado = 0;
  intentos = 0;
  ingreso = "";
  pantalla.setRGB(0, 0, 0);
  digitosIngresados = 0;
}

void setup() {
  pantalla.begin(16, 2);
  pantalla.clear();
  pinMode(boton, INPUT);

  inicializar();

}

void loop() {
  //leemos el keypad
  char tecla = teclado.getKey();

  //leer el botón
  valorBoton = digitalRead(boton);

  //checamos si el usuario desea rendirse ajustando el botón y #
  if (valorBoton == 1){
    if (tecla && tecla == '#'){
      pantalla.clear();
      pantalla.setRGB(255, 0, 0);
      estado = 4;
    }
  }

  if (estado == 0){
    pantalla.setCursor(0, 0);
    pantalla.print("Hi & Low");
    pantalla.setCursor(0, 1);
    pantalla.print("Press B to begin");

    if (tecla && tecla =='B'){
      //generamos el número escodido
      randomSeed(millis()/100/analogRead(0));
      hiddenNumber = random(1000);

      pantalla.clear();
      estado = 1;
    }

  }else if (estado == 1){
    pantalla.setCursor(0, 0);
    pantalla.print("Enter number");
    pantalla.setCursor(0, 1);

    if (tecla && tecla == 'A' && digitosIngresados > 0){
      //pasamos al siguiente estado
      userNumber = ingreso.toInt();
      pantalla.clear();
      intentos++;
      estado = 2;

    }else if (tecla && tecla == 'C' && digitosIngresados > 0){
      //borrar el número ingresado
      ingreso="";
      digitosIngresados = 0;
      pantalla.clear();

    }else if (tecla && tecla !='A' && tecla !='B' && tecla !='C' && tecla !='D' && tecla !='*' && tecla !='#' && digitosIngresados < 3){
      ingreso += tecla;
      digitosIngresados++;
      pantalla.print(ingreso);
    }

  }else if (estado == 2){
     pantalla.setCursor(0, 0);
     //comparamos el número ingresado con el número escondido
     if (userNumber > hiddenNumber){
       pantalla.print(userNumber);
       pantalla.print(" ");
       pantalla.print("TOO HIGH");

     }else if (userNumber < hiddenNumber){
       pantalla.print(userNumber);
       pantalla.print(" ");
       pantalla.print("TOO LOW");

     }else if (userNumber == hiddenNumber){
       pantalla.clear();
       estado = 3;
     }

     pantalla.setCursor(0, 1);
     pantalla.print("intentos: ");
     pantalla.print(intentos);

     //hay que presionar la tecla B para volver a adivinar
     if (tecla && tecla == 'B'){
       //regresamos al estado 1
       pantalla.clear();
       digitosIngresados = 0;
       ingreso = "";
       estado = 1;
     }
   

  }else if (estado == 3){
    pantalla.setRGB(0, 255, 0);
    pantalla.setCursor(0, 0);
    pantalla.print("Yes!! ");
    pantalla.print(hiddenNumber);
    pantalla.setCursor(0, 1);
    pantalla.print("intentos: ");
    pantalla.print(intentos);

    //hay que presionar la tecla B para volver a jugar
    if (tecla && tecla == 'B'){
      pantalla.clear();
      inicializar();
    }

  }else if (estado == 4){
    pantalla.setCursor(0, 0);
    pantalla.print("Ohh... ");
    pantalla.print(hiddenNumber);
    pantalla.setCursor(0, 1);
    pantalla.print("intentos: ");
    pantalla.print(intentos);

    //hay que presionar la tecla B para volver a jugar
    if (tecla && tecla == 'B'){
      pantalla.clear();
      inicializar();
    }
  }

  delay(200);
}
